@@ -139,11 +139,13 @@ extern ULong amd64g_calculate_sse_pmovmskb ( ULong w64hi, ULong w64lo );
 
 /* --- DIRTY HELPERS --- */
 
-extern ULong amd64g_loadF80le  ( ULong/*addr*/ );
+extern ULong amd64g_dirtyhelper_loadF80le  ( ULong/*addr*/ );
 
-extern void  amd64g_storeF80le ( ULong/*addr*/, ULong/*data*/ );
+extern void  amd64g_dirtyhelper_storeF80le ( ULong/*addr*/, ULong/*data*/ );
 
-extern void amd64g_dirtyhelper_CPUID ( VexGuestAMD64State* st );
+extern void  amd64g_dirtyhelper_CPUID ( VexGuestAMD64State* st );
+
+extern void  amd64g_dirtyhelper_FINIT ( VexGuestAMD64State* );
 
 extern ULong amd64g_dirtyhelper_RDTSC ( void );
 
@@ -153,8 +155,6 @@ extern ULong amd64g_dirtyhelper_RDTSC ( void );
 
 //extern void  amd64g_dirtyhelper_FSAVE ( VexGuestAMD64State*, HWord );
 
-//extern void  amd64g_dirtyhelper_FINIT ( VexGuestAMD64State* );
-
 //extern VexEmWarn
 //            amd64g_dirtyhelper_FRSTOR ( VexGuestAMD64State*, HWord );
 
