@@ -469,11 +469,23 @@ typedef
       /* binary */
       Iop_Add32Fx4, Iop_Sub32Fx4, Iop_Mul32Fx4, Iop_Div32Fx4, 
       Iop_Max32Fx4, Iop_Min32Fx4,
+      /* Note: For the following compares, the ppc32 front-end assumes a
+         nan in a lane of either argument returns zero for that lane. */
       Iop_CmpEQ32Fx4, Iop_CmpLT32Fx4, Iop_CmpLE32Fx4, Iop_CmpUN32Fx4, 
+      Iop_CmpGT32Fx4, Iop_CmpGE32Fx4,
 
       /* unary */
       Iop_Recip32Fx4, Iop_Sqrt32Fx4, Iop_RSqrt32Fx4,
 
+      /* --- Int to/from FP conversion --- */
+      /* Unlike the standard fp conversions, these irops take no
+         rounding mode argument. Instead the irop trailers _R{M,P,N,Z}
+         indicate the mode: {-inf, +inf, nearest, zero} respectively. */
+      Iop_I32UtoFx4,     Iop_I32StoFx4,       /* I32x4 -> F32x4       */
+      Iop_QFtoI32Ux4_RZ, Iop_QFtoI32Sx4_RZ,   /* F32x4 -> I32x4       */
+      Iop_RoundF32x4_RM, Iop_RoundF32x4_RP,   /* round to fp integer  */
+      Iop_RoundF32x4_RN, Iop_RoundF32x4_RZ,   /* round to fp integer  */
+
       /* --- 32x4 lowest-lane-only scalar FP --- */
 
       /* In binary cases, upper 3/4 is copied from first operand.  In
@@ -531,47 +543,61 @@ typedef
       Iop_NotV128,
       Iop_AndV128, Iop_OrV128, Iop_XorV128, 
 
+      /* VECTOR SHIFT (shift amt :: Ity_I8) */
+      Iop_ShlV128, Iop_ShrV128,
+
       /* MISC (vector integer cmp != 0) */
       Iop_CmpNEZ8x16, Iop_CmpNEZ16x8, Iop_CmpNEZ32x4, Iop_CmpNEZ64x2,
 
       /* ADDITION (normal / unsigned sat / signed sat) */
       Iop_Add8x16,   Iop_Add16x8,   Iop_Add32x4,  Iop_Add64x2,
-      Iop_QAdd8Ux16, Iop_QAdd16Ux8,
-      Iop_QAdd8Sx16, Iop_QAdd16Sx8,
+      Iop_QAdd8Ux16, Iop_QAdd16Ux8, Iop_QAdd32Ux4,
+      Iop_QAdd8Sx16, Iop_QAdd16Sx8, Iop_QAdd32Sx4,
 
       /* SUBTRACTION (normal / unsigned sat / signed sat) */
       Iop_Sub8x16,   Iop_Sub16x8,   Iop_Sub32x4,  Iop_Sub64x2,
-      Iop_QSub8Ux16, Iop_QSub16Ux8,
-      Iop_QSub8Sx16, Iop_QSub16Sx8,
+      Iop_QSub8Ux16, Iop_QSub16Ux8, Iop_QSub32Ux4,
+      Iop_QSub8Sx16, Iop_QSub16Sx8, Iop_QSub32Sx4,
 
       /* MULTIPLICATION (normal / high half of signed/unsigned) */
       Iop_Mul16x8,
-      Iop_MulHi16Ux8,
-      Iop_MulHi16Sx8,
+      Iop_MulHi16Ux8, Iop_MulHi32Ux4,
+      Iop_MulHi16Sx8, Iop_MulHi32Sx4,
+      /* (widening signed/unsigned of even lanes, with lowest lane=zero) */
+      Iop_MullEven8Ux16, Iop_MullEven16Ux8,
+      Iop_MullEven8Sx16, Iop_MullEven16Sx8,
 
       /* AVERAGING: note: (arg1 + arg2 + 1) >>u 1 */
-      Iop_Avg8Ux16,
-      Iop_Avg16Ux8,
+      Iop_Avg8Ux16, Iop_Avg16Ux8, Iop_Avg32Ux4,
+      Iop_Avg8Sx16, Iop_Avg16Sx8, Iop_Avg32Sx4,
 
       /* MIN/MAX */
-      Iop_Max16Sx8,
-      Iop_Max8Ux16,
-      Iop_Min16Sx8,
-      Iop_Min8Ux16,
+      Iop_Max8Sx16, Iop_Max16Sx8, Iop_Max32Sx4,
+      Iop_Max8Ux16, Iop_Max16Ux8, Iop_Max32Ux4,
+      Iop_Min8Sx16, Iop_Min16Sx8, Iop_Min32Sx4,
+      Iop_Min8Ux16, Iop_Min16Ux8, Iop_Min32Ux4,
 
       /* COMPARISON */
       Iop_CmpEQ8x16,  Iop_CmpEQ16x8,  Iop_CmpEQ32x4,
       Iop_CmpGT8Sx16, Iop_CmpGT16Sx8, Iop_CmpGT32Sx4,
+      Iop_CmpGT8Ux16, Iop_CmpGT16Ux8, Iop_CmpGT32Ux4,
 
       /* VECTOR x SCALAR SHIFT (shift amt :: Ity_I8) */
-      Iop_ShlN16x8, Iop_ShlN32x4, Iop_ShlN64x2,
-      Iop_ShrN16x8, Iop_ShrN32x4, Iop_ShrN64x2,
-      Iop_SarN16x8, Iop_SarN32x4,
+      Iop_ShlN8x16, Iop_ShlN16x8, Iop_ShlN32x4, Iop_ShlN64x2,
+      Iop_ShrN8x16, Iop_ShrN16x8, Iop_ShrN32x4, Iop_ShrN64x2,
+      Iop_SarN8x16, Iop_SarN16x8, Iop_SarN32x4,
+
+      /* VECTOR x VECTOR SHIFT / ROTATE */
+      Iop_Shl8x16, Iop_Shl16x8, Iop_Shl32x4,
+      Iop_Shr8x16, Iop_Shr16x8, Iop_Shr32x4,
+      Iop_Sar8x16, Iop_Sar16x8, Iop_Sar32x4,
+      Iop_Rol8x16, Iop_Rol16x8, Iop_Rol32x4,
 
       /* NARROWING -- narrow 2xV128 into 1xV128, hi half from left arg */
-      Iop_QNarrow16Ux8,
-      Iop_QNarrow16Sx8,
-      Iop_QNarrow32Sx4,
+      /* Note: the 16{U,S} and 32{U,S} are the pre-narrow lane widths. */
+      Iop_QNarrow16Ux8, Iop_QNarrow32Ux4,
+      Iop_QNarrow16Sx8, Iop_QNarrow32Sx4,
+      Iop_Narrow16x8, Iop_Narrow32x4,
 
       /* INTERLEAVING -- interleave lanes from low or high halves of
          operands.  Most-significant result lane is from the left
@@ -579,7 +605,17 @@ typedef
       Iop_InterleaveHI8x16, Iop_InterleaveHI16x8,
       Iop_InterleaveHI32x4, Iop_InterleaveHI64x2,
       Iop_InterleaveLO8x16, Iop_InterleaveLO16x8, 
-      Iop_InterleaveLO32x4, Iop_InterleaveLO64x2
+      Iop_InterleaveLO32x4, Iop_InterleaveLO64x2,
+
+      /* DUPLICATING -- copy value to all lanes */
+      Iop_Dup8x16, Iop_Dup16x8, Iop_Dup32x4,
+
+      /* PERMUTING -- copy src bytes to dst,
+         as indexed by control vector bytes:
+            for i in 0 .. 15 . result[i] = argL[ argR[i] ] 
+         argR[i] values may only be in the range 0 .. 15, else behaviour
+         is undefined. */
+      Iop_Perm8x16
    }
    IROp;
 
@@ -811,12 +847,18 @@ typedef
       Ijk_Call,           /* guest is doing a call */
       Ijk_Ret,            /* guest is doing a return */
       Ijk_ClientReq,      /* do guest client req before continuing */
-      Ijk_Syscall,        /* do guest syscall before continuing */
       Ijk_Yield,          /* client is yielding to thread scheduler */
       Ijk_EmWarn,         /* report emulation warning before continuing */
       Ijk_NoDecode,       /* next instruction cannot be decoded */
       Ijk_MapFail,        /* Vex-provided address translation failed */
-      Ijk_TInval          /* Invalidate translations before continuing. */
+      Ijk_TInval,         /* Invalidate translations before continuing. */
+      /* Unfortunately, various guest-dependent syscall kinds.  They
+	 all mean: do a syscall before continuing. */
+      Ijk_Sys_syscall,    /* amd64 'syscall', ppc32 'sc' */
+      Ijk_Sys_int32,      /* amd64/x86 'int $0x20' */
+      Ijk_Sys_int128,     /* amd64/x86 'int $0x80' */
+      Ijk_Sys_sysenter    /* x86 'sysenter'.  guest_EIP becomes 
+                             invalid at the point this happens. */
    }
    IRJumpKind;
 
